#ifndef OPENALPR_BENCHMARKUTILS_H
#define OPENALPR_BENCHMARKUTILS_H

#include <iostream>

std::vector<std::string> filterByExtension(std::vector<std::string> fileList, std::string extension);


#endif // OPENALPR_BENCHMARKUTILS_H
