#ifndef OPENALPR_VERSION_H
#define OPENALPR_VERSION_H

#include <stdio.h>
#include <string.h>

/*
 * return 1 if v1 > v2
 * return 0 if v1 = v2
 * return -1 if v1 < v2
 */

int cmpVersion(const char *v1, const char *v2);


#endif	/* OPENALPR_VERSION_H */

