#pragma once

#include "alpr.h"
#include "prewarp.h"

#include <stdlib.h>
#include <string.h>
#include <windows.h>

#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc.hpp"

#using <mscorlib.dll>
#include <msclr\marshal_cppstd.h>

#include "helper-net.h"
#include "bitmapmat-net.h"
#include "motiondetector-net.h"
#include "config-net.h"

namespace openalprnet {

}
