/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class com_openalpr_jni_Alpr */

#ifndef _Included_com_openalpr_jni_Alpr
#define _Included_com_openalpr_jni_Alpr
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     com_openalpr_jni_Alpr
 * Method:    initialize
 * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_openalpr_jni_Alpr_initialize
  (JNIEnv *, jobject, jstring, jstring, jstring);

/*
 * Class:     com_openalpr_jni_Alpr
 * Method:    dispose
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_com_openalpr_jni_Alpr_dispose
  (JNIEnv *, jobject);

/*
 * Class:     com_openalpr_jni_Alpr
 * Method:    is_loaded
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_com_openalpr_jni_Alpr_is_1loaded
  (JNIEnv *, jobject);

/*
 * Class:     com_openalpr_jni_Alpr
 * Method:    native_recognize
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_openalpr_jni_Alpr_native_1recognize__Ljava_lang_String_2
  (JNIEnv *, jobject, jstring);

/*
 * Class:     com_openalpr_jni_Alpr
 * Method:    native_recognize
 * Signature: ([B)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_openalpr_jni_Alpr_native_1recognize___3B
  (JNIEnv *, jobject, jbyteArray);

/*
 * Class:     com_openalpr_jni_Alpr
 * Method:    set_default_region
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_openalpr_jni_Alpr_set_1default_1region
  (JNIEnv *, jobject, jstring);

/*
 * Class:     com_openalpr_jni_Alpr
 * Method:    detect_region
 * Signature: (Z)V
 */
JNIEXPORT void JNICALL Java_com_openalpr_jni_Alpr_detect_1region
  (JNIEnv *, jobject, jboolean);

/*
 * Class:     com_openalpr_jni_Alpr
 * Method:    set_top_n
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_com_openalpr_jni_Alpr_set_1top_1n
  (JNIEnv *, jobject, jint);

/*
 * Class:     com_openalpr_jni_Alpr
 * Method:    get_version
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_openalpr_jni_Alpr_get_1version
  (JNIEnv *, jobject);

#ifdef __cplusplus
}
#endif
#endif
